#!/usr/bin/env python3

import sys
import os
import string
import numpy as np
import h5py
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt

def print_usage():
    print(f"Usage: '{os.path.basename(__file__)} <ngauge output file> <counter ID> [<counter ID> ...]'", file=sys.stderr)


def fail():
    print_usage()
    sys.exit(1)


def list_datasets(file):
    with h5py.File(file,'r') as f:
        print(f"The following counters are available in file {os.path.basename(file)}:", file=sys.stderr)
        for k in f.keys():
            print(hex(int(k)))


def display(file, counters):
    fig = plt.figure()
    units = set()
    with h5py.File(file,'r') as f:
        for c in counters:
            dset = f[c]
            unit = dset.attrs["unit"]
            units.add(unit)

        units = list(units)
        gs = fig.add_gridspec(len(units), hspace=0)
        axs = gs.subplots(sharex=True)
        for c in counters:
            dset = f[c]
            unit = dset.attrs["unit"]
            name = dset.attrs["name"]
            start_time = (dset[1:, 0] - dset[1, 0]) * 1e-9
            end_time = (dset[1:, 1] - dset[1, 0]) * 1e-9
            time = (start_time + end_time) / 2.0
            time_error = (end_time - start_time) / 2.0
            # TODO: proper normalization - TBD if from the metadata or not
            if "PCI" not in name:
                normalizer = (np.diff(dset[:, 0] - dset[0, 0])) * 1e-9
            else:
                normalizer = np.ones_like(dset[1:, 2])

            data = dset[1:, 2] / normalizer
            normalizer_error = time_error / normalizer
            data_error = dset[1:, 2] * normalizer_error / normalizer

            try:
                axs[units.index(unit)].errorbar(time, data, xerr=time_error, yerr=data_error, marker='x', label=f"{name} ({hex(int(c))})", linestyle='none')
                axs[units.index(unit)].legend()
            except TypeError:
                axs.errorbar(time, data, xerr=time_error, yerr=data_error, marker='x', label=f"{name} ({hex(int(c))})", linestyle='none')
                axs.legend()

    #axs[0].set_yscale('log', base=2)  # Uncomment per plot, if needed
    fig.supylabel('Counts')
    plt.xlabel('Time (s)')
    plt.show()


if __name__ == '__main__':
    if len(sys.argv) < 3:
        fail()

    if not os.path.isfile(sys.argv[1]):
        fail()

    counters = []
    for cnt in sys.argv[2:]:
        if cnt.isdigit():
            counters.append(cnt)
        elif cnt.startswith("0x") and all(c in string.hexdigits for c in cnt[2:]):
            counters.append(str(int(cnt, 16)))
        else:
            fail()

    try:
        display(sys.argv[1], counters)
    except KeyError:
        list_datasets(sys.argv[1])
        sys.exit(1)
